@@ -1,5 +1,5 @@
 from os import path
-from fitsnap3.io.sections.sections import Section
+from .sections import Section
 
 
 class Outfile(Section):
@@ -9,20 +9,20 @@ class Outfile(Section):
         self._check_relative()
         self._outfile()
         self.output_style = self.get_value("OUTFILE", "output_style", "ORIGINAL")
+        self.print_a = self.get_value("OUTFILE", "print_a", "False", "bool")
         self.delete()
 
     def _outfile(self):
-        self.config_file = self.get_value("OUTFILE", "configs", "fitsnap_configs.pkl.gz")
-        self._check_path(self.config_file)
-        self.metric_file = self.get_value("OUTFILE", "metrics", "fitsnap_metrics.csv")
-        self._check_path(self.metric_file)
-        self.potential_name = self.get_value("OUTFILE", "potential", "fitsnap_potential")
-        self._check_path(self.potential_name)
+        self.config_file = self._check_path(self.get_value("OUTFILE", "configs", "fitsnap_configs.pkl.gz"))
+        self.metric_file = self._check_path(self.get_value("OUTFILE", "metrics", "fitsnap_metrics.csv"))
+        self.potential_name = self._check_path(self.get_value("OUTFILE", "potential", "fitsnap_potential"))
+        self.detailed_errors_file = \
+            self._check_path(self.get_value("OUTFILE", "detailed_errors", "fitsnap_detailed_errors.dat"))
         return
 
     def _check_relative(self):
         if self._args.relative:
-            self.base_path, _ = path.split(self._args.infile)
+            self.base_path = self._get_relative_directory(self)
         else:
             self.base_path = None
 
@@ -35,5 +35,6 @@ class Outfile(Section):
             return name
         names = [name, name + '.snapparam', name + '.snapcoeff']
         for element in names:
-            if self._args.overwrite and path.exists(element):
-                raise FileExistsError(f"File {element} already exists.")
\ No newline at end of file
+            if not self._args.overwrite and path.exists(element):
+                raise FileExistsError(f"File {element} already exists.")
+        return name
