@@ -1,4 +1,4 @@
-from fitsnap3.io.sections.sections import Section
+from .sections import Section
 
 
 class Reference(Section):
@@ -9,6 +9,9 @@ class Reference(Section):
         self.atom_style = self.get_value("REFERENCE", "atom_style", "atomic").lower()
         self.lmp_pairdecl = []
         self.lmp_pairdecl.append("pair_style " + self.get_value("REFERENCE", "pair_style", "zero 10.0"))
+        if not config.has_section("REFERENCE"):
+            self.delete()
+            return
         for name, value in self._config.items("REFERENCE"):
             if not name.find("pair_coeff"):
                 self.lmp_pairdecl.append("pair_coeff " + value)
