@@ -37,6 +37,9 @@ from psutil import virtual_memory
 from itertools import chain
 import ctypes
 import signal
+from inspect import isclass
+from pkgutil import iter_modules
+from importlib import import_module
 try:
     # stubs = 0 MPI is active
     stubs = 0
@@ -49,12 +52,18 @@ def printf(*args, **kw):
     print(*args, flush=True)
 
 
+class GracefulError(BaseException):
+
+    def __init__(self, *args, **kwargs):
+        pass
+
+
 class GracefulKiller:
 
-    def __init__(self, comm, failsafe=False):
+    def __init__(self, comm):
         self._comm = comm
         self._rank = 0
-        self.failsafe = failsafe
+        self.already_killed = False
         if self._comm is not None:
             self._rank = self._comm.Get_rank()
             signal.signal(signal.SIGINT, self.exit_gracefully)
@@ -63,9 +72,9 @@ class GracefulKiller:
     def exit_gracefully(self, signum, frame):
         if self._rank == 0:
             printf("attempting to exit gracefully")
-            printf("exiting from exit code", signum, "at", frame)
-        if self.failsafe:
+        if self.already_killed:
             self._comm.Abort()
+        raise GracefulError("exiting from exit code", signum, "at", frame)
 
 
 def _rank_zero(method):
@@ -137,14 +146,14 @@ class ParallelTools:
             self._node_index = 0
             self._number_of_nodes = 1
 
-        killer = GracefulKiller(self._comm)
+        self.killer = GracefulKiller(self._comm)
 
         self._comm_split()
         self._lmp = None
         self._seed = 0.0
         self._set_seed()
         self.shared_arrays = {}
-        self._calculator_options = {}
+        self.fitsnap_dict = {}
         self.logger = None
 
     @stub_check
@@ -175,6 +184,9 @@ class ParallelTools:
     def get_rank(self):
         return self._rank
 
+    def get_subrank(self):
+        return self._sub_rank
+
     @_rank_zero
     def single_print(self, *args, **kw):
         printf(*args)
@@ -244,13 +256,31 @@ class ParallelTools:
         else:
             raise TypeError("name must be a string")
 
+    @_sub_rank_zero
+    def add_2_fitsnap(self, name, an_object):
+
+        if isinstance(name, str):
+            if name in self.fitsnap_dict:
+                raise NameError("name is already in dictionary")
+            self.fitsnap_dict[name] = an_object
+        else:
+            raise TypeError("name must be a string")
+
+    @stub_check
+    def _comm_fitsnap(self, name):
+
+        if self._sub_rank == 0:
+            if name not in self.fitsnap_dict:
+                raise NameError("Dictionary element not yet in fitsnap_dictionary")
+        else:
+            self.fitsnap_dict[name] = None
+
+        self.fitsnap_dict[name] = self._sub_comm.bcast(self.fitsnap_dict[name])
+
     @stub_check
     def allgather(self, array):
         return self._head_group_comm.allgather(array)
 
-    def add_calculator_option(self, option, boolean):
-        self._calculator_options[option] = boolean
-
     @stub_check
     def all_barrier(self):
         self._comm.Barrier()
@@ -281,6 +311,18 @@ class ParallelTools:
         else:
             raise TypeError("Parallel tools cannot split {} within node.".format(obj))
 
+    def check_lammps(self, lammps_noexceptions=0):
+        cmds = ["-screen", "none", "-log", "none"]
+        if stubs == 0:
+            self._lmp = lammps(comm=self._micro_comm, cmdargs=cmds)
+        else:
+            self._lmp = lammps(cmdargs=cmds)
+
+        if not (self._lmp.has_exceptions or lammps_noexceptions):
+            raise Exception("Fitting interrupted! LAMMPS not compiled with C++ exceptions handling enabled")
+        self._lmp.close()
+        self._lmp = None
+
     def initialize_lammps(self, lammpslog=0, printlammps=0):
         cmds = ["-screen", "none"]
         if not lammpslog:
@@ -302,7 +344,7 @@ class ParallelTools:
             self._lmp = None
         return self._lmp
 
-    def slice_array(self, name, num_types=None):
+    def slice_array(self, name):
         if name in self.shared_arrays:
             if name != 'a':
                 s = slice(self._sub_rank, None, self._sub_size)
@@ -315,13 +357,17 @@ class ParallelTools:
     def slice_a(self, name='a'):
         nof = len(pt.shared_arrays["number_of_atoms"].array)
         s = slice(self._sub_rank, nof, self._sub_size)
+        if self._sub_rank != 0:
+            self._comm_fitsnap("a_indices")
+            self.fitsnap_dict["a_indices"] = self.fitsnap_dict["a_indices"][s]
+            return
         indices = [0]
         self.shared_arrays[name].group_index = []
         self.shared_arrays[name].group_energy_index = []
         self.shared_arrays[name].group_force_index = []
         self.shared_arrays[name].group_stress_index = []
         count = [0]
-        for i, value in enumerate(self.shared_arrays['files_per_group'].array):
+        for i, value in enumerate(self.shared_arrays['configs_per_group'].array):
             count.append(count[i]+value)
         j = 0
         e_temp = []
@@ -331,46 +377,48 @@ class ParallelTools:
         for i in range(nof):
             if i in count:
                 self.shared_arrays[name].group_index.append(j)
-                if self._calculator_options["energy"] and i > 0:
+                if self.fitsnap_dict["energy"] and i > 0:
                     self.shared_arrays[name].group_energy_index.append(e_temp)
                     e_temp = []
-                if self._calculator_options["force"] and i > 0:
+                if self.fitsnap_dict["force"] and i > 0:
                     self.shared_arrays[name].group_force_index.append(f_temp)
                     f_temp = []
-                if self._calculator_options["stress"] and i > 0:
+                if self.fitsnap_dict["stress"] and i > 0:
                     self.shared_arrays[name].group_stress_index.append(s_temp)
                     s_temp = []
-            if self._calculator_options["energy"]:
+            if self.fitsnap_dict["energy"]:
                 e_temp.append(j)
                 j += 1
-            if self._calculator_options["force"]:
+            if self.fitsnap_dict["force"]:
                 f_temp.append(j)
                 j += 3 * pt.shared_arrays["number_of_atoms"].array[i]
-            if self._calculator_options["stress"]:
+            if self.fitsnap_dict["stress"]:
                 s_temp.append(j)
                 j += 6
             indices.append(j)
             atoms.append(pt.shared_arrays["number_of_atoms"].array[i])
-        if self._calculator_options["energy"]:
+        if self.fitsnap_dict["energy"]:
             self.shared_arrays[name].group_energy_index.append(e_temp)
             self.shared_arrays[name].energy_index = \
                 list(chain.from_iterable(self.shared_arrays[name].group_energy_index))
             self.shared_arrays[name].group_energy_length = \
                 sum(len(row) for row in self.shared_arrays[name].group_energy_index)
-        if self._calculator_options["force"]:
+        if self.fitsnap_dict["force"]:
             self.shared_arrays[name].group_force_index.append(f_temp)
             self.shared_arrays[name].force_index = \
                 list(chain.from_iterable(self.shared_arrays[name].group_force_index))
             self.shared_arrays[name].group_force_length = \
                 sum(len(row) for row in self.shared_arrays[name].group_force_index)
-        if self._calculator_options["stress"]:
+        if self.fitsnap_dict["stress"]:
             self.shared_arrays[name].group_stress_index.append(s_temp)
             self.shared_arrays[name].stress_index = \
                 list(chain.from_iterable(self.shared_arrays[name].group_stress_index))
             self.shared_arrays[name].group_stress_length = \
                 sum(len(row) for row in self.shared_arrays[name].group_stress_index)
         self.shared_arrays[name].group_index.append(j)
-        self.shared_arrays[name].indices = indices[s]
+        self.add_2_fitsnap("a_indices", indices)
+        self._comm_fitsnap("a_indices")
+        self.fitsnap_dict["a_indices"] = indices[s]
         self.shared_arrays[name].num_atoms = atoms
 
     @stub_check
@@ -392,14 +440,37 @@ class ParallelTools:
         self._comm.Abort()
 
     def exception(self, err):
+        self.killer.already_killed = True
+
         if pt.logger is None and self._rank == 0:
             raise err
 
         pt.close_lammps()
         if self._rank == 0:
             self.logger.exception(err)
+
         sleep(5)
-        self.abort()
+        if self._comm is not None:
+            self.abort()
+
+    # Where The Object Oriented Magic Happens
+    # from files in this_file's directory import subclass of this_class
+    @staticmethod
+    def get_subclasses(this_name, this_file, this_class):
+        # Reset Path cls to remove old cls paths
+        from pathlib import Path
+
+        name = this_name.split('.')
+        package_dir = Path(this_file).resolve().parent
+        for (_, module_name, c) in iter_modules([package_dir]):
+            if module_name != name[-1] and module_name != name[-2]:
+                module = import_module(f"{'.'.join(name[:-1])}.{module_name}")
+                for attribute_name in dir(module):
+                    attribute = getattr(module, attribute_name)
+
+                    if isclass(attribute) and issubclass(attribute, this_class) and attribute is not this_class:
+                        # Add the class to this package's variables
+                        globals()[attribute_name] = attribute
 
 
 class SharedArray:
@@ -479,7 +550,7 @@ class Output:
         pt.exception(err)
 
 
-if __name__ == "fitsnap3.parallel_tools":
+if __name__.split(".")[-1] == "parallel_tools":
     pt = ParallelTools()
     if stubs == 0:
         double_size = MPI.DOUBLE.Get_size()
