@@ -1,5 +1,5 @@
-from fitsnap3.io.input import config
-from fitsnap3.parallel_tools import pt
+from ..io.input import config
+from ..parallel_tools import pt
 import numpy as np
 from pandas import DataFrame
 
@@ -25,6 +25,14 @@ class Solver:
     def fit_gather(self):
         self.all_fits = pt.allgather(self.fit)
 
+    def extras(self):
+        if config.sections["SOLVER"].dump_a:
+            self._dump_a()
+        if config.sections["SOLVER"].dump_x:
+            self._dump_x()
+        if config.sections["SOLVER"].dump_b:
+            self._dump_b()
+
     def _offset(self):
         num_types = config.sections["BISPECTRUM"].numtypes
         if num_types > 1:
@@ -35,11 +43,13 @@ class Solver:
         else:
             self.fit = np.insert(self.fit, 0, 0)
 
-    def error_analysis(self, data):
+    def error_analysis(self):
         for option in ["Unweighted", "Weighted"]:
             self.weighted = option
             self._all_error()
-            self._group_error(data)
+            self._group_error()
+        #   Print of errors per config
+        #     self._config_error()
 
         if self.template_error is True:
             self._template_error()
@@ -47,7 +57,7 @@ class Solver:
         self.errors = DataFrame.from_records(self.errors)
         self.errors = self.errors.set_index(["Group", "Weighting", "Subsystem", ]).sort_index()
 
-        if config.sections["CALCULATOR"].bzeroflag:
+        if config.sections["CALCULATOR"].calculator == "LAMMPSSNAP" and config.sections["BISPECTRUM"].bzeroflag:
             self._offset()
 
     def _all_error(self):
@@ -60,31 +70,48 @@ class Solver:
         self._combined()
 
     def _energy(self):
-        testing = -1 * pt.shared_arrays['files_per_group'].testing
+        testing = -1 * pt.shared_arrays['configs_per_group'].testing
         a, b, w = self._make_abw(pt.shared_arrays['a'].energy_index, 1)
         self._errors([[0, testing]], ['*ALL'], "Energy", a, b, w)
         if testing != 0:
             self._errors([[testing, 0]], ['*ALL'], "Energy_testing", a, b, w)
 
     def _force(self):
-        testing = -1 * pt.shared_arrays['files_per_group'].testing
-        a, b, w = self._make_abw(pt.shared_arrays['a'].force_index, pt.shared_arrays['a'].num_atoms)
+        num_forces = np.array(pt.shared_arrays['a'].num_atoms)*3
+        if pt.shared_arrays['configs_per_group'].testing:
+            testing = -1 * np.sum(num_forces[-pt.shared_arrays['configs_per_group'].testing:])
+        else:
+            testing = 0
+        a, b, w = self._make_abw(pt.shared_arrays['a'].force_index, num_forces.tolist())
+        # print out predicted vs true forces
+        # detailed_errors = 1
+        # if detailed_errors and self.weighted == "Unweighted":
+        #     from csv import writer
+        #     true, pred = b, a @ self.fit
+        #     with open('detailed_errors.dat', 'w') as f:
+        #         writer = writer(f, delimiter=' ')
+        #         writer.writerows(zip(true, pred, true-pred))
+
         self._errors([[0, testing]], ['*ALL'], "Force", a, b, w)
         if testing != 0:
             self._errors([[testing, 0]], ['*ALL'], "Force_testing", a, b, w)
 
     def _stress(self):
-        testing = -1 * pt.shared_arrays['files_per_group'].testing
+        testing = -6 * pt.shared_arrays['configs_per_group'].testing
         a, b, w = self._make_abw(pt.shared_arrays['a'].stress_index, 6)
         self._errors([[0, testing]], ['*ALL'], "Stress", a, b, w)
         if testing != 0:
             self._errors([[testing, 0]], ['*ALL'], "Stress_testing", a, b, w)
 
+    def _combined(self):
+        self._errors([[0, pt.shared_arrays["configs_per_group"].testing_elements]], ["*ALL"], "Combined")
+        if pt.shared_arrays["configs_per_group"].testing_elements != 0:
+            self._errors([[pt.shared_arrays["configs_per_group"].testing_elements, 0]], ['*ALL'], "Combined_testing")
+
     @staticmethod
     def _make_abw(type_index, buffer):
         if isinstance(buffer, list):
             length = sum(buffer)
-            length *= 3
         else:
             length = len(type_index) * buffer
         width = np.shape(pt.shared_arrays['a'].array)[1]
@@ -94,7 +121,7 @@ class Solver:
         i = 0
         for j, value in enumerate(type_index):
             if isinstance(buffer, list):
-                spacing = buffer[j]*3
+                spacing = buffer[j]
             else:
                 spacing = buffer
             a[i:i+spacing] = pt.shared_arrays['a'].array[value:value+spacing]
@@ -103,21 +130,11 @@ class Solver:
             i += spacing
         return a, b, w
 
-    def _combined(self):
-        testing = -1 * pt.shared_arrays['files_per_group'].testing
-        self._errors([[0, testing]], ["*ALL"], "Combined")
-        if testing != 0:
-            self._errors([[testing, 0]], ['*ALL'], "Combined_testing")
-
-    def _group_error(self, data):
+    def _group_error(self):
         groups = []
-        testing = len(data)-pt.shared_arrays['files_per_group'].testing
-        for i, file in enumerate(data):
-            if i < testing:
-                groups.append(file["Group"])
-            else:
-                groups.append(file["Group"]+'_Testing')
-        groups = sorted(set(groups))
+        for group in pt.shared_arrays['configs_per_group'].list:
+            group = group.split('/')[-1]
+            groups.append(group)
         if config.sections["CALCULATOR"].energy:
             self._group_energy(groups)
         if config.sections["CALCULATOR"].force:
@@ -144,6 +161,13 @@ class Solver:
         index, a, b, w = self._make_group_abw(group_index, length, 6)
         self._errors(index, groups, "Stress", a, b, w)
 
+    def _group_combined(self, groups):
+        index = []
+        group_index = pt.shared_arrays['a'].group_index
+        for i in range(len(group_index) - 1):
+            index.append([group_index[i], group_index[i+1]])
+        self._errors(index, groups, "Combined")
+
     @staticmethod
     def _make_group_abw(group_index, length, buffer):
         index = []
@@ -172,12 +196,58 @@ class Solver:
             index.append(temp)
         return index, a, b, w
 
-    def _group_combined(self, groups):
-        index = []
-        group_index = pt.shared_arrays['a'].group_index
-        for i in range(len(group_index) - 1):
-            index.append([group_index[i], group_index[i+1]])
-        self._errors(index, groups, "Combined")
+    def _config_error(self):
+        config_index = 0
+        current_index = 0
+        for i, num_atoms in enumerate(pt.shared_arrays['a'].num_atoms):
+            this_config = pt.shared_arrays['number_of_atoms'].configs[i]
+            if isinstance(this_config, str):
+                this_config = this_config.split('/')[-2] + ':' + this_config.split('/')[-1]
+                this_config = [this_config]
+            elif isinstance(this_config, list):
+                this_config = [this_config[-1].split('/')[-1] + ':' + str(this_config[0])]
+            if config.sections["CALCULATOR"].energy:
+                # current index is the index of the top of energy
+                self._config_energy(this_config, current_index)
+                current_index += 1
+            if config.sections["CALCULATOR"].force:
+                # current index is the index of the top of force
+                self._config_force(this_config, current_index, 3*num_atoms)
+                current_index += 3*num_atoms
+            if config.sections["CALCULATOR"].stress:
+                self._config_stress(this_config, current_index)
+                current_index += 6
+            self._config_combined(this_config, config_index, current_index)
+            config_index = current_index
+
+    def _config_energy(self, this_config, current_index):
+        index, a, b, w = self._make_config_abw(current_index, 1)
+        self._errors(index, this_config, "Energy", a, b, w)
+
+    def _config_force(self, this_config, current_index, length):
+        index, a, b, w = self._make_config_abw(current_index, length)
+        self._errors(index, this_config, "Force", a, b, w)
+
+    def _config_stress(self, this_config, current_index):
+        index, a, b, w = self._make_config_abw(current_index, 6)
+        self._errors(index, this_config, "Stress", a, b, w)
+
+    def _config_combined(self, this_config, config_index, current_index):
+        index, a, b, w = self._make_config_abw(config_index, current_index-config_index)
+        self._errors(index, this_config, "Combined", a, b, w)
+
+    @staticmethod
+    def _make_config_abw(i, buffer):
+        index = None
+        width = np.shape(pt.shared_arrays['a'].array)[1]
+        a = np.zeros((buffer, width))
+        b = np.zeros((buffer,))
+        w = np.zeros((buffer,))
+        a[:] = pt.shared_arrays['a'].array[i:i + buffer]
+        b[:] = pt.shared_arrays['b'].array[i:i + buffer]
+        w[:] = pt.shared_arrays['w'].array[i:i + buffer]
+
+        return index, a, b, w
 
     def _errors(self, index, category, gtype, a=None, b=None, w=None):
         if a is None:
@@ -187,7 +257,11 @@ class Solver:
         if w is None:
             w = pt.shared_arrays['w'].array
         for i, group in enumerate(category):
-            if index[i][1] != 0:
+            if index is None:
+                a_err = a
+                b_err = b
+                w_err = w
+            elif index[i][1] != 0:
                 a_err = a[index[i][0]:index[i][1]]
                 b_err = b[index[i][0]:index[i][1]]
                 w_err = w[index[i][0]:index[i][1]]
@@ -203,6 +277,9 @@ class Solver:
                 nconfig = len(pred)
             res = true - pred
             mae = np.sum(np.abs(res) / nconfig)
+            # relative mae
+            # rres = ((true+1)-pred)/(true+1)
+            # rel_mae = np.sum(np.abs(rres) / nconfig)
             mean_dev = np.sum(np.abs(true - np.median(true)) / nconfig)
             ssr = np.square(res).sum()
             mse = ssr / nconfig
@@ -226,3 +303,12 @@ class Solver:
 
     def _template_error(self):
         pass
+
+    def _dump_a(self):
+        raise NotImplementedError("This method is either not implemented or solver is not a linear solver")
+
+    def _dump_x(self):
+        raise NotImplementedError("This method is either not implemented or solver is not a linear solver")
+
+    def _dump_b(self):
+        raise NotImplementedError("This method is either not implemented or solver is not a linear solver")
\ No newline at end of file
